=== modified file 'NEWS'
--- NEWS	2005-10-17 22:31:39 +0000
+++ NEWS	2005-10-17 23:13:00 +0000
@@ -16,25 +16,25 @@
       Goffredo Baroncelli.
     
     * 'bzr check' now accepts -v for verbose reporting, and checks for
-      ghosts in the branch.
+      ghosts in the branch. (Robert Collins)
 
     * New command 're-sign' which will regenerate the gpg signature for 
-      a revision.
+      a revision. (Robert Collins)
 
     * If you set check_signatures=require for a path in 
       ~/.bazaar/branches.conf then bzr will invoke your
       gpg_signing_command (defaults to gpg) and record a digital signature
-      of your commit.
+      of your commit. (Robert Collins)
 
   TESTING:
 
     * The 'bzr selftest --pattern' option for has been removed, now 
       test specifiers on the command line can be simple strings, or 
-      regexps, or both.
+      regexps, or both. (Robert Collins)
 
     * Passing -v to selftest will now show the time each test took to 
       complete, which will aid in analysing performance regressions and
-      related questions.
+      related questions. (Robert Collins)
 
     * 'bzr selftest' runs all tests, even if one feels, unless '--one'
       is given.
@@ -71,7 +71,11 @@
 
     * Two decorators, needs_read_lock and needs_write_lock have been added
       to the branch module. Use these to cause a function to run in a
-      read or write lock respectively.
+      read or write lock respectively. (Robert Collins)
+
+    * Branch.open_containing now returns a tuple (Branch, relative-path),
+      which allows direct access to the common case of 'get me this file
+      from its branch'. (Robert Collins)
 
 
 bzr 0.1.1 2005-10-12

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-10-16 22:31:25 +0000
+++ bzrlib/add.py	2005-10-17 23:13:00 +0000
@@ -66,7 +66,7 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch.open_containing(file_list[0])
+    b = Branch.open_containing(file_list[0])[0]
     return smart_add_branch(b, file_list, recurse, reporter)
 
         

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2005-10-17 05:18:20 +0000
+++ bzrlib/annotate.py	2005-10-17 23:13:00 +0000
@@ -92,7 +92,7 @@
     from bzrlib.workingtree import WorkingTree
 
     enable_default_logging()
-    b = Branch.open_containing(sys.argv[1])
+    b = Branch.open_containing(sys.argv[1])[0]
     tree = WorkingTree(b.base, b)
     rp = tree.relpath(sys.argv[1])
     tree = b.revision_tree(b.last_revision())

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-17 22:31:39 +0000
+++ bzrlib/branch.py	2005-10-17 23:13:00 +0000
@@ -125,11 +125,12 @@
 
         Basically we keep looking up until we find the control directory or
         run into the root.  If there isn't one, raises NotBranchError.
+        If there is one, it is returned, along with the unused portion of url.
         """
         t = get_transport(url)
         while True:
             try:
-                return _Branch(t)
+                return _Branch(t), t.relpath(url)
             except NotBranchError:
                 pass
             new_t = t.clone('..')

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 11:56:54 +0000
+++ bzrlib/builtins.py	2005-10-17 23:13:00 +0000
@@ -79,15 +79,17 @@
     
     def run(self, all=False, show_ids=False, file_list=None, revision=None):
         if file_list:
-            b = Branch.open_containing(file_list[0])
-            tree = WorkingTree(b.base, b)
-            file_list = [tree.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
+            b, relpath = Branch.open_containing(file_list[0])
+            if relpath == '' and len(file_list) == 1:
                 file_list = None
+            else:
+                # generate relative paths.
+                # note that if this is a remote branch, we would want
+                # relpath against the transport. RBC 20051018
+                tree = WorkingTree(b.base, b)
+                file_list = [tree.relpath(x) for x in file_list]
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
             
         from bzrlib.status import show_status
         show_status(b, show_unchanged=all, show_ids=show_ids,
@@ -111,7 +113,7 @@
             raise BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id is None and revision is None:
             raise BzrCommandError('You must supply either --revision or a revision_id')
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision_id is not None:
             sys.stdout.write(b.get_revision_xml_file(revision_id).read())
         elif revision is not None:
@@ -127,7 +129,7 @@
 
     This is equal to the number of revisions on this branch."""
     def run(self):
-        print Branch.open_containing('.').revno()
+        print Branch.open_containing('.')[0].revno()
 
 
 class cmd_revision_info(Command):
@@ -147,7 +149,7 @@
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
 
         for rev in revs:
             revinfo = rev.in_history(b)
@@ -205,7 +207,7 @@
         for d in dir_list:
             os.mkdir(d)
             if not b:
-                b = Branch.open_containing(d)
+                b = Branch.open_containing(d)[0]
             b.add([d])
             print 'added', d
 
@@ -216,8 +218,8 @@
     hidden = True
     
     def run(self, filename):
-        branch = Branch.open_containing(filename)
-        print WorkingTree(branch.base, branch).relpath(filename)
+        branch, relpath = Branch.open_containing(filename)
+        print relpath
 
 
 class cmd_inventory(Command):
@@ -225,7 +227,7 @@
     takes_options = ['revision', 'show-ids']
     
     def run(self, revision=None, show_ids=False):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision is None:
             inv = b.read_working_inventory()
         else:
@@ -251,7 +253,7 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
 
         # TODO: glob expansion on windows?
         tree = WorkingTree(b.base, b)
@@ -275,7 +277,7 @@
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         tree = WorkingTree(b.base, b)
         b.rename_one(tree.relpath(from_name), tree.relpath(to_name))
 
@@ -297,7 +299,7 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = Branch.open_containing(names_list[0])
+        b = Branch.open_containing(names_list[0])[0]
         tree = WorkingTree(b.base, b)
         rel_names = [tree.relpath(x) for x in names_list]
         
@@ -338,7 +340,7 @@
         from shutil import rmtree
         import errno
         
-        br_to = Branch.open_containing('.')
+        br_to = Branch.open_containing('.')[0]
         stored_loc = br_to.get_parent()
         if location is None:
             if stored_loc is None:
@@ -410,7 +412,7 @@
         try:
             br_from.setup_caching(cache_root)
             if basis is not None:
-                basis_branch = Branch.open_containing(basis)
+                basis_branch = Branch.open_containing(basis)[0]
             else:
                 basis_branch = None
             if len(revision) == 1 and revision[0] is not None:
@@ -452,7 +454,7 @@
     takes_args = ['dir?']
 
     def run(self, dir='.'):
-        b = Branch.open_containing(dir)
+        b = Branch.open_containing(dir)[0]
         old_inv = b.basis_tree().inventory
         new_inv = b.read_working_inventory()
 
@@ -468,7 +470,7 @@
     
     def run(self, branch=None):
         import info
-        b = Branch.open_containing(branch)
+        b = Branch.open_containing(branch)[0]
         info.show_info(b)
 
 
@@ -483,7 +485,7 @@
     aliases = ['rm']
     
     def run(self, file_list, verbose=False):
-        b = Branch.open_containing(file_list[0])
+        b = Branch.open_containing(file_list[0])[0]
         tree = WorkingTree(b.base, b)
         b.remove([tree.relpath(f) for f in file_list], verbose=verbose)
 
@@ -498,9 +500,8 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = Branch.open_containing(filename)
-        tree = WorkingTree(b.base, b)
-        i = b.inventory.path2id(tree.relpath(filename))
+        b, relpath = Branch.open_containing(filename)
+        i = b.inventory.path2id(relpath)
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
         else:
@@ -515,10 +516,9 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)
         inv = b.inventory
-        tree = WorkingTree(b.base, b)
-        fid = inv.path2id(tree.relpath(filename))
+        fid = inv.path2id(relpath)
         if fid == None:
             raise BzrError("%r is not a versioned file" % filename)
         for fip in inv.get_idpath(fid):
@@ -529,7 +529,7 @@
     """Display list of revision ids on this branch."""
     hidden = True
     def run(self):
-        for patchid in Branch.open_containing('.').revision_history():
+        for patchid in Branch.open_containing('.')[0].revision_history():
             print patchid
 
 
@@ -545,7 +545,7 @@
 class cmd_directories(Command):
     """Display list of versioned directories in this branch."""
     def run(self):
-        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
+        for name, ie in Branch.open_containing('.')[0].read_working_inventory().directories():
             if name == '':
                 print '.'
             else:
@@ -602,14 +602,14 @@
         from bzrlib.diff import show_diff
 
         if file_list:
-            b = Branch.open_containing(file_list[0])
+            b = Branch.open_containing(file_list[0])[0]
             tree = WorkingTree(b.base, b)
             file_list = [tree.relpath(f) for f in file_list]
             if file_list == ['']:
                 # just pointing to top-of-tree
                 file_list = None
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
 
         if revision is not None:
             if len(revision) == 1:
@@ -638,7 +638,7 @@
     # level of effort but possibly much less IO.  (Or possibly not,
     # if the directories are very large...)
     def run(self, show_ids=False):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         old = b.basis_tree()
         new = b.working_tree()
         for path, ie in old.inventory.iter_entries():
@@ -655,7 +655,7 @@
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         td = compare_trees(b.basis_tree(), b.working_tree())
 
         for path, id, kind, text_modified, meta_modified in td.modified:
@@ -667,7 +667,7 @@
     """List files added in working tree."""
     hidden = True
     def run(self):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         wt = b.working_tree()
         basis_inv = b.basis_tree().inventory
         inv = wt.inventory
@@ -689,7 +689,7 @@
     takes_args = ['filename?']
     def run(self, filename=None):
         """Print the branch root."""
-        b = Branch.open_containing(filename)
+        b = Branch.open_containing(filename)[0]
         print b.base
 
 
@@ -732,15 +732,13 @@
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = Branch.open_containing(filename)
-            tree = WorkingTree(b.base, b)
-            fp = tree.relpath(filename)
-            if fp:
+            b, fp = Branch.open_containing(filename)
+            if fp != '':
                 file_id = b.read_working_inventory().path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = Branch.open_containing('.')
+            b, relpath = Branch.open_containing('.')
             file_id = None
 
         if revision is None:
@@ -793,10 +791,9 @@
     hidden = True
     takes_args = ["filename"]
     def run(self, filename):
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)[0]
         inv = b.read_working_inventory()
-        tree = WorkingTree(b.base, b)
-        file_id = inv.path2id(tree.relpath(filename))
+        file_id = inv.path2id(relpath)
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
             print "%6d %s" % (revno, what)
 
@@ -807,7 +804,7 @@
     # TODO: Take a revision or remote path and list that tree instead.
     hidden = True
     def run(self, revision=None, verbose=False):
-        b = Branch.open_containing('.')
+        b, relpath = Branch.open_containing('.')[0]
         if revision == None:
             tree = b.working_tree()
         else:
@@ -825,7 +822,7 @@
     """List unknown files."""
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in Branch.open_containing('.').unknowns():
+        for f in Branch.open_containing('.')[0].unknowns():
             print quotefn(f)
 
 
@@ -855,7 +852,7 @@
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = Branch.open_containing('.')
+        b, relpath = Branch.open_containing('.')
         ifn = b.abspath('.bzrignore')
 
         if os.path.exists(ifn):
@@ -895,7 +892,7 @@
 
     See also: bzr ignore"""
     def run(self):
-        tree = Branch.open_containing('.').working_tree()
+        tree = Branch.open_containing('.')[0].working_tree()
         for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
@@ -919,7 +916,7 @@
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print Branch.open_containing('.').get_rev_id(revno)
+        print Branch.open_containing('.')[0].get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -938,7 +935,7 @@
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision is None:
             rev_id = b.last_revision()
         else:
@@ -977,9 +974,8 @@
             raise BzrCommandError("bzr cat requires a revision number")
         elif len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = Branch.open_containing('.')
-        tree = WorkingTree(b.base, b)
-        b.print_file(tree.relpath(filename), revision[0].in_history(b).revno)
+        b, relpath = Branch.open_containing(filename)
+        b.print_file(relpath, revision[0].in_history(b).revno)
 
 
 class cmd_local_time_offset(Command):
@@ -1027,7 +1023,7 @@
         from bzrlib.status import show_status
         from cStringIO import StringIO
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         tree = WorkingTree(b.base, b)
         if selected_list:
             selected_list = [tree.relpath(s) for s in selected_list]
@@ -1075,7 +1071,7 @@
 
     def run(self, dir='.', verbose=False):
         from bzrlib.check import check
-        check(Branch.open_containing(dir), verbose)
+        check(Branch.open_containing(dir)[0], verbose)
 
 
 class cmd_scan_cache(Command):
@@ -1120,7 +1116,7 @@
     
     def run(self, email=False):
         try:
-            b = bzrlib.branch.Branch.open_containing('.')
+            b = bzrlib.branch.Branch.open_containing('.')[0]
             config = bzrlib.config.BranchConfig(b)
         except NotBranchError:
             config = bzrlib.config.GlobalConfig()
@@ -1212,8 +1208,8 @@
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = Branch.open_containing(branch)
-        branch2 = Branch.open_containing(other)
+        branch1 = Branch.open_containing(branch)[0]
+        branch2 = Branch.open_containing(other)[0]
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
@@ -1277,7 +1273,7 @@
         if merge_type is None:
             merge_type = ApplyMerge3
         if branch is None:
-            branch = Branch.open_containing('.').get_parent()
+            branch = Branch.open_containing('.')[0].get_parent()
             if branch is None:
                 raise BzrCommandError("No merge location known or specified.")
             else:
@@ -1334,7 +1330,7 @@
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
             revno = revision[0].in_history(b).revno
         merge(('.', revno), parse_spec('.'),
               check_clean=False,
@@ -1342,7 +1338,7 @@
               backup_files=not no_backup,
               file_list=file_list)
         if not file_list:
-            Branch.open_containing('.').set_pending_merges([])
+            Branch.open_containing('.')[0].set_pending_merges([])
 
 
 class cmd_assert_fail(Command):
@@ -1414,7 +1410,7 @@
         if verbose and quiet:
             raise BzrCommandError('Cannot pass both quiet and verbose')
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         parent = b.get_parent()
         if remote is None:
             if parent is None:
@@ -1427,7 +1423,7 @@
             # We only update parent if it did not exist, missing
             # should not change the parent
             b.set_parent(remote)
-        br_remote = Branch.open_containing(remote)
+        br_remote = Branch.open_containing(remote)[0]
         return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
 
 
@@ -1456,7 +1452,7 @@
     takes_args = ['branch?']
     def run(self, branch='.', revision=None, long=False):
         from bzrlib.testament import Testament
-        b = Branch.open_containing(branch)
+        b = Branch.open_containing(branch)[0]
         b.lock_read()
         try:
             if revision is None:
@@ -1493,13 +1489,12 @@
 
     def run(self, filename, all=False, long=False):
         from bzrlib.annotate import annotate_file
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)
         b.lock_read()
         try:
             tree = WorkingTree(b.base, b)
-            rp = tree.relpath(filename)
             tree = b.revision_tree(b.last_revision())
-            file_id = tree.inventory.path2id(rp)
+            file_id = tree.inventory.path2id(relpath)
             file_version = tree.inventory[file_id].revision
             annotate_file(b, file_version, file_id, long, all, sys.stdout)
         finally:
@@ -1521,7 +1516,7 @@
             raise BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id is None and revision is None:
             raise BzrCommandError('You must supply either --revision or a revision_id')
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
         if revision_id is not None:
             b.sign_revision(revision_id, gpg_strategy)

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2005-10-13 07:06:01 +0000
+++ bzrlib/conflicts.py	2005-10-17 23:13:00 +0000
@@ -37,7 +37,7 @@
     files.)
     """
     def run(self):
-        for path in Branch.open_containing('.').working_tree().iter_conflicts():
+        for path in Branch.open_containing('.')[0].working_tree().iter_conflicts():
             print path
 
 class cmd_resolve(bzrlib.commands.Command):
@@ -50,7 +50,7 @@
             if not all:
                 raise BzrCommandError(
                     "command 'resolve' needs one or more FILE, or --all")
-            tree = Branch.open_containing('.').working_tree()
+            tree = Branch.open_containing('.')[0].working_tree()
             file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
         else:
             if all:

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-17 05:18:20 +0000
+++ bzrlib/merge.py	2005-10-17 23:13:00 +0000
@@ -199,7 +199,7 @@
             
 def get_tree(treespec, local_branch=None):
     location, revno = treespec
-    branch = Branch.open_containing(location)
+    branch = Branch.open_containing(location)[0]
     if revno is None:
         revision = None
     elif revno == -1:
@@ -264,7 +264,7 @@
     try:
         if this_dir is None:
             this_dir = '.'
-        this_branch = Branch.open_containing(this_dir)
+        this_branch = Branch.open_containing(this_dir)[0]
         this_rev_id = this_branch.last_revision()
         if this_rev_id is None:
             raise BzrCommandError("This branch has no commits")

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-10-16 14:51:21 +0000
+++ bzrlib/revisionspec.py	2005-10-17 23:13:00 +0000
@@ -313,7 +313,7 @@
     def _match_on(self, branch, revs):
         from branch import Branch
         from revision import common_ancestor, MultipleRevisionSources
-        other_branch = Branch.open_containing(self.spec)
+        other_branch = Branch.open_containing(self.spec)[0]
         revision_a = branch.last_revision()
         revision_b = other_branch.last_revision()
         for r, b in ((revision_a, branch), (revision_b, other_branch)):
@@ -339,7 +339,7 @@
     def _match_on(self, branch, revs):
         from branch import Branch
         from fetch import greedy_fetch
-        other_branch = Branch.open_containing(self.spec)
+        other_branch = Branch.open_containing(self.spec)[0]
         revision_b = other_branch.last_revision()
         if revision_b is None:
             raise NoCommits(other_branch)

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-17 11:56:54 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-17 23:13:00 +0000
@@ -821,4 +821,15 @@
         branch = Branch.open('to')
         self.assertEqual(1, len(branch.revision_history()))
 
+    def test_log(self):
+        self.build_tree(['branch/', 'branch/file'])
+        branch = Branch.initialize('branch')
+        branch.add(['file'])
+        branch.commit('add file', rev_id='A')
+        url = self.get_remote_url('branch/file')
+        output = self.capture('log %s' % url)
+        self.assertEqual(7, len(output.split('\n')))
+        
+
+
 

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-17 22:31:39 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-17 23:13:00 +0000
@@ -167,8 +167,10 @@
         self.assertRaises(NotBranchError, Branch.open_containing,
                           self.get_remote_url('g/p/q'))
         b = Branch.initialize('.')
-        Branch.open_containing(self.get_remote_url(''))
-        Branch.open_containing(self.get_remote_url('g/p/q'))
+        branch, relpath = Branch.open_containing(self.get_remote_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = Branch.open_containing(self.get_remote_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
         
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-17 00:00:33 +0000
+++ bzrlib/transport/local.py	2005-10-17 23:13:00 +0000
@@ -68,6 +68,8 @@
         """Return the local path portion from a given absolute path.
         """
         from bzrlib.osutils import relpath
+        if abspath is None:
+            abspath = '.'
         return relpath(self.base, abspath)
 
     def has(self, relpath):

=== modified file 'setup.py' (properties changed: -x to +x)
